<?php
// $Id: settings.api.inc,v 1.1.2.1 2009/07/04 18:12:23 overall Exp $

function settings_get_module() {
  static $module_list = NULL;
  
  if (!isset($module_list)) {
    $module_list = array();
    $result = db_query("SELECT name, filename FROM {system} WHERE type = 'module' AND status = 1");
    while ($module = db_fetch_object($result)) {
      $module->path = realpath(dirname($module->filename));
      $module_list[$module->path] = $module;
    }
  }
  
  $backtrace = debug_backtrace(FALSE);
  while ($call = array_shift($backtrace)) {
    $call_path = dirname($call['file']);
    do {
      $module = $module_list[$call_path];
      if (isset($module) && ($module->name != 'settings')) {
        return $module->name;
      }
      
      $prev_call_path = $call_path;
      $call_path = dirname($call_path);
    } while (($call_path != $prev_call_path) && ($call_path != '.'));
  } 
}

/**
 * Get all settings for the given module.
 * 
 * @param $module
 *   The name of the module.
 * @return
 *   Array of module settings.      
 */
function settings_obtain_settings($module) {
  /*global*/static $settings;

  if (!isset($settings[$module])) {
    //$settings[$module] = module_invoke($module, 'settings_get_settings_info');
    $settings[$module] = call_user_func($module . '_' . 'settings_get_settings_info'); // for use at bootstrap
    if (!isset($settings[$module])) {
      $settings[$module] = array();
    }
  }

  return $settings[$module];
}

/**
 * Check the existance of the given module setting.
 * 
 * @param $module
 *   The name of the module.
 * @param $name
 *   The name of the setting.
 * @return
 *   TRUE - if the given setting exists, otherwise will trigger a warning.      
 */
function settings_check($module, $name) {
  $module_settings = settings_obtain_settings($module);
  
  if (isset($module_settings['defaults'][$name])) {
    return TRUE;
  }
  else {
    trigger_error(t('Setting %name in module %module is not exist.', array('%name' => $name, '%module' => $module)), E_USER_WARNING);
  }
}

function settings_get_default_for($module, $name) {
  if (settings_check($module, $name)) {
    $module_settings = settings_obtain_settings($module);
    return $module_settings['defaults'][$name];
  }
}

function settings_get_var_for($module, $name) {
  if (settings_check($module, $name)) {
    return variable_get($module.'_'.$name, NULL);
  }
}

/**
 * Get current value of the given setting or default value (or specified value) if the setting is not set.
 * 
 * @param $module
 *   The name of the module.
 * @param $name
 *   The name of the setting.
 * @param $value
 *   Optional. The value returned if the setting is not set. If not specified defined default value will be returned. 
 * @return
 *   The current value of the setting or default value (or specified value) if the setting is not set.      
 */
function settings_get_for($module, $name, $value = NULL) {
  if (settings_check($module, $name)) {
    if (!isset($value)) {
      $module_settings = settings_obtain_settings($module);
      $value = $module_settings['defaults'][$name];
    }
    return variable_get($module.'_'.$name, $value);
  }
}

/**
 * Set current value of the given setting.
 * 
 * @param $module
 *   The name of the module.
 * @param $name
 *   The name of the setting.
 * @param $value
 *   The value to be set. 
 */
function settings_set_for($module, $name, $value) {
  if (settings_check($module, $name)) {
    variable_set($module.'_'.$name, $value);
  }
}

function settings_delete_for($module, $name) {
  variable_del($module.'_'.$name);
}

function settings_reset_for($module, $name) {
  settings_set_for($module, $name, settings_get_default_for($module, $name));
}

/**
 * Get current values of all settings for the given module or default values (or specified values) if the particular setting is not set.
 * 
 * @param $module
 *   The name of the module.
 * @param $values
 *   Optional. The array of values returned if the particular setting is not set. If not specified defined default values will be returned. 
 * @return
 *   Current values  or default values (or specified values) if the particular setting is not set.      
 */
function settings_get_all_for($module, $values = array()) {
  global $conf;

  $result = array();
  $module_settings = settings_obtain_settings($module);
  foreach ($module_settings['defaults'] as $name => $default_value) {
    $result[$name] = (array_key_exists($module.'_'.$name, $conf) ? $conf[$module.'_'.$name] : (array_key_exists($name, $values) ? $values[$name] : $default_value));
  }
  return $result;
}

function settings_set_all_for($module, $values) {
  $module_settings = settings_obtain_settings($module);
  foreach (array_keys($module_settings['defaults']) as $name) {
    if (array_key_exists($name, $values)) {
      settings_set_for($module, $name, $values[$name]);
    }
    else {
      settings_delete_for($module, $name);
    }
  }
}

/**
 * Delete all module settings. So, defined default values will be used as current values.  
 * 
 * @param $module
 *   The name of the module.
 */
function settings_delete_all_for($module) {
  $module_settings = settings_obtain_settings($module);
  foreach (array_keys($module_settings['defaults']) as $name) {
    settings_delete_for($module, $name);
  }
}

/**
 * Reset all module settings, by setting them to their current values. (Useless function...)  
 * 
 * @param $module
 *   The name of the module.
 */
function settings_reset_all_for($module) {
  $module_settings = settings_obtain_settings($module);
  foreach ($module_settings['defaults'] as $name => $value) {
    variable_set($module.'_'.$name, $value);
  }
}

/**
 * Highlight radios here because child elements of parent expanded element will descend parent attributes including "class".
 * Highlight checkboxes here because we need to highlight each child checkbox of parent checkboxes independently.  
 */
function _settings_prepare_expanded_element($element, &$form_state) {
  $module = $element['#settings_module'];
  $name = $element['#settings_name'];
  switch ($element['#type']) {
    case 'radios':
      $element = _settings_highlight_element_for($module, $name, $element);
      return $element;
    case 'checkboxes':
      $element = settings_prepare_settings_form_for($module, $element);
      return $element;
  }
}

function _settings_highlight_element_for($module, $name, $element) {
  // Highlight default values
  $default = settings_get_default_for($module, $name);
  $var = settings_get_var_for($module, $name);
  if (!isset($var) || ($var == $default)) {
    $element['#attributes']['class'] .= ' '.(in_array($element['#type'], array('checkbox', 'radio', 'textarea')) ? 'settings_use_default_parent_append' : 'settings_use_default_after');
  }
  else {
    if (settings_get_for('settings', 'show_default_value')) {
      $element['#attributes']['class'] .= ' '.(in_array($element['#type'], array('checkbox', 'radio', 'textarea')) ? 'settings_default_valuet_parent_append' : 'settings_default_value_after');
      drupal_add_js(array('settings' => array('settings_default_values' => array($module.'_'.$name => check_plain($default)))), 'setting');
    }
  }
  return $element;
}

/**
 * Prepare settings form by specifing "#default_value" values and rename element names from "<setting_name>" to "<module_name> '_' <setting_name>".
 * 
 * @param $module
 *   The name of the module.
 * @param $form
 *   The settings form to be prepared.
 * @return
 *   The prepared settings form.
 *   
 * @todo
 *   Use "element_children()" (?).        
 */
function settings_prepare_settings_form_for($module, $form, $order = TRUE, $suffix = NULL/*, $level = 0*/) {
  $result_form = array();
  
  $properties_names = element_properties($form);
  foreach ($properties_names as $name) {
    $result_form[$name] = $form[$name];
  }
  
  $elements_names = element_children($form);
  $weight = -count($elements_names);
  foreach ($elements_names as $name) {
    $element = $form[$name];
    
    // If this is a form element
    if (is_array($element)) {
      // If this is an input form element
      if (in_array($element['#type'], array('checkbox', 'checkboxes', 'date', 'file', 'password', 'radio', 'radios', 'select', 'textarea', 'textfield', 'weight', 'hidden'))) {
        // If "default_value" applicable
        if (!in_array($element['#type'], array('file', 'password'))) {
          if (!isset($suffix)) {
            $element['#default_value'] = settings_get_for($module, $name);
          }
          else {
            // Workaround for use settings module not only for modules settings, but also for filters settings
            if (settings_check($module, $name)) {
              $module_settings = settings_obtain_settings($module);
              $element['#default_value'] = variable_get($module.'_'.$name.'_'.$suffix, $module_settings['defaults'][$name]);
            }
          }
        }

        // Highlight default values
        if (!in_array($element['#type'], array('checkboxes', 'radios'))) {
          $element = _settings_highlight_element_for($module, $name, $element);
        }
        else {
          $element['#settings_module'] = $module;
          $element['#settings_name'] = $name;
          $element['#after_build'][] = '_settings_prepare_expanded_element';
        }

        // Rename element name in result form
        $new_name = $module.'_'.$name;
        if (isset($suffix)) {
          $new_name = $new_name.'_'.$suffix;
        }
        $result_form[$new_name] = $element;
        $name = $new_name;
      }
      else {
        $result_form[$name] = settings_prepare_settings_form_for($module, $element, $suffix/*, $level + 1*/);
      }

      // Set element weight
      if ($order) {
        $result_form[$name]['#weight'] = $weight; 
        $weight++;
      }
    }
  }

  /*if ($level == 0) {
    $result_form['#prefix'] = drupal_get_form('settings_export_inline_form') . drupal_get_form('settings_import_inline_form');
  }*/ 
  
  return $result_form;
}

/*function settings_export_inline_form() {
  return array(
    'export' => array(
      '#type' => 'submit',
      '#value' => t('Export'),
      '#submit' => array('settings_export_file'),
      '#weight' => 0,
    ),
  );
}*/

//define('SETTINGS_FILE_EXT', 'modules_settings.txt');

/*function settings_import_inline_form() {
  return array(
    '#attributes' => array('enctype' => 'multipart/form-data'),
    'file' => array(
      '#type' => 'file',
      '#title' => t('Settings File'),
      '#description' => t('Select file with modules settings (@file_ext) to import settings.', array('@file_ext' => '*.'.SETTINGS_FILE_EXT)),
      '#weight' => 0,
    ),
    'import' => array(
      '#type' => 'submit',
      '#value' => t('Import'),
      '#submit' => array('settings_import_file'),
      '#weight' => 1,
    ),
  );  
}*/

function settings_get_default($name) {
  return settings_get_default_for(settings_get_module(), $name);
}

function settings_get_var($name) {
  return settings_get_var_for(settings_get_module(), $name);
}

function settings_get($name, $value = NULL) {
  return settings_get_for(settings_get_module(), $name, $value);
}

function settings_set($name, $value) {
  return settings_set_for(settings_get_module(), $name, $value);
}

function settings_reset($name) {
  return settings_reset_for(settings_get_module(), $name);
}

function settings_delete($name) {
  return settings_delete_for(settings_get_module(), $name);
}

function settings_get_all($values = array()) {
  return settings_get_all_for(settings_get_module(), $values);
}

function settings_set_all($values) {
  return settings_set_all_for(settings_get_module(), $values);
}

function settings_reset_all() {
  return settings_reset_all_for(settings_get_module());
}

function settings_delete_all() {
  return settings_delete_all_for(settings_get_module());
}

function settings_prepare_settings_form($form, $suffix = NULL) {
  return settings_prepare_settings_form_for(settings_get_module(), $form, $suffix);
}
